"use strict"
var
  MostCreate= require( "@most/create")

function MostRAF( options){
	var
	  requestAnimationFrame= options&& options.requestAnimationFrame|| defaults.requestAnimationFrame,
	  cancelAnimationFrame= options&& options.cancelAnimationFrame|| defaults.cancelAnimationFrame,
	  timeout= options&& options.timeout, // if no RAF happen in this time, switch to background mode
	  tick= options&& options.tick|| 16 // background rate in ms

	if( !requestAnimationFrame){
		throw new Error( "requestAnimationFrame not found")
	}
	return new MostCreate(function( add, end, error){
		var
		  requestId, // handle for requestAnimationFrame
		  timeoutId, // handle for timer checking whether to switch into background mode
		  backgroundId // handle for background interval

		// frame adds a frame sequence
		function frame( sequence){
			requestId= requestAnimationFrame({ frame: sequence})
			if( timeout){
				if( timeoutId){
					// clear previous running timeout
					clearTimeout( timeoutId)
				}
				if( !backgroundId){
					// clear running background interval
					clearInterval( backgroundId)
				}
				timeoutId= setTimeout( startBackground, timeout)
			}
			add( sequence)
		}

		// background triggered
		function background(){
		}

		// begin background polling
		function startBackground(){
			// we've already timed out so fire now
			background()
			// start interal at tickrate
			backgroundId= setInterval( background, tick)
		}

		function dispose(){
			if( requestId&& cancelAnimationFrame){
				cancelAnimationFrame( requestId)
			}
			if( timeoutId){
				clearTimeout( timeoutId)
			}
			if( backgroundId){
				clearInterval( backgroundId)
			}
		}

		requestAnimationFrame( frame)
		return dispose
	})
}

var hasWindow= typeof window!== "undefined"
var defaults= {
	requestAnimationFrame: hasWindow? window.requestAnimationFrame: null,
	cancelAnimationFrame: hasWindow? window.cancelAnimationFrame: null
}

module.exports= MostRAF
module.exports.MostRAF= MostRAF
module.exports.defaults= defaults
